// Filename	: RefCountBuffer.h
// Purpose	: A reference counting buffer. Use for fast and efficient way to create buffers
//            and pass them between your functions.
// Author	: Erad Fridman
// Date		: 15/11/2002

#ifndef __REFCOUNT_BUFFER_H__
#define __REFCOUNT_BUFFER_H__

#include "refcount.h"

class RefCountBufferImp : public RefCountObj {
public:
	friend class RefCountBuffer;

private:
	RefCountBufferImp (unsigned int size, const char *buffer);
	~RefCountBufferImp ();

private:
	char *_buf;
	unsigned int _size;
};

//---------------------------------------------------------------------------------------
class RefCountBuffer {
public:
	// c'tors & d'tor
	RefCountBuffer (unsigned int size = 0, const char *buffer = NULL);
	virtual ~RefCountBuffer();
	RefCountBuffer (const RefCountBuffer &other);

	// operator = overload
	RefCountBuffer & operator = (const RefCountBuffer &other);

	// RefCountBuffer methods
	char *get_data ()
	{
		return _imp -> _buf;
	}

	const char *get_data () const
	{
		return _imp -> _buf;
	}

	unsigned int get_size () const
	{
		return _imp -> _size;
	}

	RefCountBuffer clone() const
	{
		return RefCountBuffer(_imp->_size, _imp->_buf);
	}

private:
	RefCountPtr <RefCountBufferImp> _imp;
};

RefCountBuffer operator + (const RefCountBuffer &x, const RefCountBuffer &y);

#endif // __REFCOUNT_BUFFER_H__