// Sync.obj
/*---------------------------------------------------------------------------*/
#ifndef __SYNC_OBJ_H__
#define __SYNC_OBJ_H__
/*---------------------------------------------------------------------------*/
#include <windows.h>
#include <winbase.h>
#include <assert.h>
/*---------------------------------------------------------------------------*/
/** Base class for object that require syncronized access
  * ( Mutial exclusive access )
  * every object of this type supports 2 synchronization methods :
  * RequestObj() - Wait until the object is free ( no timeout )
  * ReleaseObj() - Free the object for other users.
  * It is implemented using a Win32 Mutex object.
  */
class SyncObj 
{
public:
	/** Default Ctor (creates mutex) */
	SyncObj() 
	{
		// create private mutex object ( not owned )
		p_hMutex = CreateMutex(NULL, FALSE, NULL );		
		assert(p_hMutex);
	};
	/** Dtor (destroys mutex) */
	~SyncObj() 
	{
		// WARNING : Destroy the object only if you are sure that
		// there are'nt any pending requests.
		CloseHandle(p_hMutex);
	};

	/** Request mutex */
	void RequestObj()
	{
		// block calling thread, until the mutex is signaled
		WaitForSingleObject(p_hMutex, INFINITE);
	};

	/** Release mutex */
	void ReleaseObj()
	{
		ReleaseMutex(p_hMutex);
	};

private:
	/** The mutex */
	HANDLE p_hMutex;
};
/*---------------------------------------------------------------------------*/
#endif // defined __SYNC_OBJ_H__