// agentlist.cpp
// The agentlist class implementation
/*---------------------------------------------------------------------------*/
#pragma warning(disable : 4786) 
#include "agentlist.h"
using namespace std;
/*---------------------------------------------------------------------------*/
// CLASS CLIENT
void Client::write (ofstream &os) const
{	
	os.write(_nickname,NICKNAME_SIZE);
	os.write((char *)&_ip,sizeof(_ip));
	os.write((char *)&_port,sizeof(_port));
	os.write(_public_key,PUBLIC_KEY_SIZE);
}

void Client::read (ifstream &is)
{
	memset(_nickname,'\0',NICKNAME_SIZE);
	is.read(_nickname,NICKNAME_SIZE);
	is.read((char *)&_ip,sizeof(_ip));
	is.read((char *)&_port,sizeof(_port));
	is.read(_public_key,PUBLIC_KEY_SIZE);
}
/*---------------------------------------------------------------------------*/
// CLASS AGENTLIST
bool AgentList::updateClient (const Client &c)
{
	key k;
	list_iter iter;
	bool new_agent=false;

	k = makeKey(c.getIp(),c.getPort());
	iter = _w.find(k);

	// If agent is not present in both R and W lists, it means it's a register
	// request (otherwise a get list request)
	if ((_r.find(k)==_r.end()) && (iter == _w.end()))
		new_agent = true;

	// If agent exist, delete it (for update)
	if (iter != _w.end())
		_w.erase(k);

	// Enter agent to w list
	_w[k] = c;

	return new_agent;
}
	
void AgentList::removeClient (const Client &c)
{
	key k;

	k = makeKey(c.getIp(),c.getPort());
	_w.erase(k);
}

void AgentList::updateList ()
{
	// Perform switching!
	_r.clear();
	_r = _w;
	_w.clear();
}

void AgentList::write (ofstream &os) const
{
	list_const_iter iter;
	int s=_r.size(); // Size of r list
	
	// Write size of list
	os.write((char *)&s,sizeof(int));
	// Write list
	for (iter = _r.begin(); iter != _r.end(); iter++)
		iter->second.write(os);
}
		
void AgentList::read (ifstream &is)
{
	int size; // Size of list on file
	int i;	
	Client c;
	key k;
	
	// Read size of list
	is.read((char *)&size,sizeof(size));
	// Read list (to R)
	for (i=0; i<size;i++)
	{
		c.read(is);
		k = makeKey(c.getIp(),c.getPort());
		_r[k] = c;
	}
}

const AgentList::agent_list   AgentList::ComputeQuited () const
{
	agent_list quited;
	list_const_iter iter;

	quited.clear();
	for(iter=_r.begin();iter!=_r.end();iter++)
		// If the agent is in R and not in W, its down or quited
		if (_w.find(iter->first)==_w.end())
			quited[iter->first]=iter->second;
	return quited;
}
/*---------------------------------------------------------------------------*/