// agentlist.h
// the agentlist class
/*---------------------------------------------------------------------------*/
#ifndef CLIENT_H
#define CLIENT_H
/*---------------------------------------------------------------------------*/
#include <string>
#include <map>
#include <iostream>
#include <fstream>
#include "protocol.h"
using namespace std;
/*---------------------------------------------------------------------------*/
/**
 * This class holds basic client data
 */
class Client
{
private:
	/** Client's nick */
	char _nickname[12]; 
	/** Client's IP */
	unsigned int _ip;	
	/** Client's port */
	unsigned short _port; 
	/** Client's public key */
	char _public_key[PUBLIC_KEY_SIZE]; 
public:

	/** Default Ctor */
	Client () {}
	

	/** The Ctor makes a client from an AGENT */
	Client (const struct AGENT &agent)
	{
		memcpy(_nickname , agent.nickname,12);
		_ip = agent.address.ip;
		_port = agent.address.port;
		memcpy(_public_key,agent.pubkey,PUBLIC_KEY_SIZE);
	}

	/** The Ctor makes a client from an ADDRESS */
	Client (const struct ADDRESS &adr)
	{
		_ip = adr.ip;
		_port = adr.port;
	}
	
	/** The Ctor makes a client from file */
	Client (ifstream &is) {read(is); }

	/** Dtor */
	~Client () {}

	/** Nick accessor */
	string getNick () const {return _nickname; }
	
	/** IP accessor */
	int getIp () const {return _ip; }
	
	/** PORT accessor */
	short getPort () const {return _port; }
	
	/** PublicKey accessor */
	const char * getPublicKey () const {return _public_key; }
	
	/** IP set function */
	int setIp (const int ip) {_ip = ip; }
	
	/** PORT set function */
	short setPort (const short port) {_port = port; }

	/** Writes client to a file */
	void write (ofstream &os) const;
	
	/** Reads client from a file */
	void read (ifstream &is);
};

/**
  * This class holds the server's agents lists 
  * and includes methods for handling them, perform switch etc
  */
class AgentList
{
public:
	/** Map key (IP&PORT) */
	typedef string key; 
	/** Agent list */
	typedef map<key,Client> agent_list; 
	/** Iterator for agent list */
	typedef map<key,Client>::iterator list_iter; 
	/** Const iterator for agent list */
	typedef map<key,Client>::const_iterator list_const_iter; 	
private:
	map<key,Client> _r, _w; /** The agents R and W lists */

public:
	/** Default Ctor */
	AgentList () {};
	/** The Ctor reads data (R list) from file (recovery) */
	AgentList (ifstream &is) {read(is); }
	/** Dtor */
	~AgentList () {}

	/** Returns a copy of the R list */
	agent_list getList () const {return _r; }
	
	/** Updates a client in W list
	  * Returns TRUE if its a new client 
	  */
	bool updateClient (const Client &c);
	
	/** Removes a client from W list */
	void removeClient (const Client &c);
	
	/** Updates R list with W list, and cleans W */
	void updateList ();
	
	/** Writes data to flie (recovery) */
	void write (ofstream &os) const;
	
	/** Reads data from file (recovery) */
	void read (ifstream &is);
	
	/** Returns a list of agents that issued "quit" request */
	const agent_list   ComputeQuited () const;
	
	/** Returns a list of agents exist in system
	  * Note: should be used only before switching lists
	  */
	const agent_list   ComputeExist  () const { return _w;}
	
private:
	/** Makes a map key for a client */
	key makeKey (int ip, short port) 
	{
		string temp;
	
		char ipp[100]; 
		sprintf(ipp,"%d%d'\0'",ip,port); 
		temp = ipp;
		return temp;
	}
};
/*---------------------------------------------------------------------------*/
#endif