// log.cpp
// Log class implementation
/*---------------------------------------------------------------------------*/
#include <windows.h>
#include "log.h"
#include "agentlist.h"
#include "SyncObj.h"
/*---------------------------------------------------------------------------*/
void Log::WriteRegister (struct ADDRESS adr,bool new_agent)
{
	// Critical section
	_file_lock.RequestObj();

	WriteTimeDate();
	if (new_agent)
		_log<<"Event: register ";
	else
		_log<<"Event: get-list ";

	_log<<"Client data: ";
	_log<<"IP: "<<convert(adr.ip)<<" " ;
	_log<<"PORT: "<<adr.port<<" ";
	_log<<endl;

	_file_lock.ReleaseObj();
	// End of critical section
}

void Log::WriteQuits(AgentList::list_const_iter quited_agents_begin,
					 AgentList::list_const_iter quited_agents_end)
{
	
	if (quited_agents_begin==quited_agents_end)
		return;	
	
	// Critical section
	_file_lock.RequestObj();
	
	WriteTimeDate();

	for(;quited_agents_begin!=quited_agents_end;quited_agents_begin++)
	{
		_log<<"Event: Agent is down ";
		_log<<"Client data: ";
		_log<<"IP: "<<convert(quited_agents_begin->second.getIp())<<" " ;
		_log<<"PORT: "<<quited_agents_begin->second.getPort()<<" ";
		_log<<endl;
	}

	_file_lock.ReleaseObj();
	// End of critical section


}

void Log::WriteExist(AgentList::list_const_iter exist_agents_begin,
					 AgentList::list_const_iter exist_agents_end)
{
	
	
	// Critical section
	_file_lock.RequestObj();
	
	WriteTimeDate();

	if (exist_agents_begin==exist_agents_end)
	{
		_log<<"No agents in system"<<endl;
		_file_lock.ReleaseObj();
		return;	
	}
	
	_log<<"Event : The following agents are in the system"<<endl;
	
	for(;exist_agents_begin!=exist_agents_end;exist_agents_begin++)
	{
		_log<<"Client data: ";
		_log<<"IP: "<<convert(exist_agents_begin->second.getIp())<<" " ;
		_log<<"PORT: "<<exist_agents_begin->second.getPort()<<" ";
		_log<<endl;
	}

	_file_lock.ReleaseObj();
	// End of critical section


}


void Log::WriteUnRegister (struct ADDRESS adr)
{
	// Critical section
	_file_lock.RequestObj();

	WriteTimeDate();
	_log<<"Event: quit ";
	_log<<"Client data: ";
	_log<<"IP: "<<convert(adr.ip)<<" " ;
	_log<<"PORT: "<<adr.port<<" ";
	_log<<endl;

	_file_lock.ReleaseObj();
	// End of critical section
}

void Log::WriteServerStart()
{
	WriteMsg("SERVER IS UP");
}

void Log::WriteServerEnd ()
{
	WriteMsg("SERVER IS DOWN");
}

void Log::WriteSwitch ()
{
	WriteMsg("SERVER IS DOING SWITCHING");
}

void Log::WriteTimeDate()
{
	time_t aclock; // Current time
	time( &aclock );                
	
	// Make current time readble
	string tmp(asctime( localtime( &aclock ))); 

	// Last char of asctime is '\n', remove it
	tmp[tmp.size()-1] = ' ';
	_log<<tmp;
}

void Log::WriteMsg (const string &msg)
{
	// Critical section
	_file_lock.RequestObj();

	WriteTimeDate();

	_log<<msg<<endl;

	_file_lock.ReleaseObj();
	// End of critical section
}

char *Log::convert(unsigned int ip)
{
	in_addr ina;
	ina.S_un.S_addr = ip;
	return inet_ntoa(ina);
}
/*---------------------------------------------------------------------------*/