// log.h
// Server's log handling class
/*---------------------------------------------------------------------------*/
#ifndef LOG_H
#define LOG_H
/*---------------------------------------------------------------------------*/
#pragma warning(disable : 4786) 
#include "agentlist.h"
#include <fstream>
#include <string>
#include "SyncObj.h"
using namespace std;
/*---------------------------------------------------------------------------*/
/**
 * Logs server activity and agentlist info
 */
class Log
{
private:
	/**Log file */
	ofstream _log; 
	/** Lock for the log file*/
	SyncObj _file_lock; 

public:
	/** Default constructor - opens log.txt*/
	Log ()
	{
		_log.open("log.txt",ios::ate|ios::app);
	}
	/** Constructor - opens specified file as log*/
	Log (const string &file)
	{
		_log.open(file.c_str(),ios::ate|ios::app);
	}
	/** Destructor */
	~Log()
	{
		_log.close();
	}

	// Write events to log methods:
	/** Writes register (get-list) event*/
	void WriteRegister (struct ADDRESS adr,bool new_agent=false);
	/** Writes unregister event*/
	void WriteUnRegister (struct ADDRESS adr);
	/** Writes server-start event*/
	void WriteServerStart ();
	/** Writes server-end event*/
	void WriteServerEnd ();
	/** Writes server-switching event*/
	void WriteSwitch ();
	/**Writes which agents exist in system*/
	void WriteQuits(AgentList::list_const_iter quited_agents_begin,
					AgentList::list_const_iter quited_agents_end);
	/** Writes existing agents to log file*/ 
	void WriteExist(AgentList::list_const_iter exist_agents_begin,
					AgentList::list_const_iter exist_agents_end);

private:
	/** Writes msg to log*/
	void WriteMsg (const string &msg);
	/** Writes time and date to log*/
	void WriteTimeDate ();
	/** Converts ip number to dotted representation*/
	char * convert(unsigned int ip);

};
/*---------------------------------------------------------------------------*/
#endif








