// protocol.h
// Header file for protocol definitions
/*---------------------------------------------------------------------------*/
#ifndef _PROTOCOL_H
#define _PROTOCOL_H
/*---------------------------------------------------------------------------*/
#include "time.h"
/*---------------------------------------------------------------------------*/
#pragma pack (push, 1)

// Server's default port
const int PORT=7878; 

// Magic number for communication 
// Assaf A, Erad F, Oded l & Amir Hadidi 
const unsigned int	MAGIC_HEADER = 0xAAEF01AA; 

// Public key buffer definition 
const int PUBLIC_KEY_SIZE = 399;
typedef char PUBLIC_KEY[PUBLIC_KEY_SIZE];

// Nickname buffer definition
const int NICKNAME_SIZE = 12;

typedef char NICKNAME[NICKNAME_SIZE];

/** Agent address structure */
struct ADDRESS {
	/** Agent's IP */
	unsigned int ip; 
	/** Agent's PORT */
	unsigned short port;
};

/** A player (agent) information structure */
struct AGENT {
	/** Agent's Address (Ip + Port) */
	ADDRESS		address; 
	/** Agent's public-key */
	PUBLIC_KEY	pubkey;
	/** Agent's nick */
	NICKNAME	nickname;
};

/** Parameters of center (which we send to agent) */
struct CENTER_PARAMS {
	/** Length of msg-send route */
	long route_length;
	/** Global time */
	time_t center_time;
	/** Period of time in which agents can send msg */
	time_t tick_time;	
	/** Period of time in which agents must finish reply */
	time_t reply_time;
	/** Period of time in which agents should request list*/
	time_t keepalive_time; 
	/** Time when agents can use the new list */
	time_t  list_use_time;
};

// Header codes for recieved buffer
const unsigned char	REGISTER_CODE	= 0x00; // Not in use
const unsigned char UNREGISTER_CODE	= 0xFF;
const unsigned char MESSAGE_CODE	= 0x11;
const unsigned char REPLY_CODE		= 0x22;
const unsigned char GET_AGENTS_CODE	= 0x33;
const unsigned char STATISTICS_CODE = 0x55; // Currently not in use either

const int HEADER_SIZE = 1; // size of protocol header
const int MAGIC_SIZE = 4; // size of magic number in the protocol
const int MAX_REQUEST_SIZE = 422; // size of max request from agent
const int NUMBER_OF_AGENTS_SIZE = 4; // size of number of number of agents
const unsigned int QUIT_REQUEST_SIZE=
						HEADER_SIZE+MAGIC_SIZE+sizeof(struct ADDRESS);
					// size of quit request from agents

#pragma pack (pop)

/*---------------------------------------------------------------------------*/
#endif /* _PROTOCOL_H */