// server.cpp
// Implementation of the server class
/*---------------------------------------------------------------------------*/
#pragma warning(disable : 4786) 

#include "agentlist.h"
#include "protocol.h"
#include <stdlib.h>
#include <time.h>
#include <winsock2.h>
#include <stdio.h>
#include <iostream>
#include <fstream>
#include "RefCountBuffer.h"
#include "server.h"
#include "servererror.h"
#include "handleclient.h"
#include <conio.h>
#include "SyncObj.h"
/*---------------------------------------------------------------------------*/
void HandleScreenThread(LPVOID lpParameter) 
{
	char x;
	Server *s = (Server *)lpParameter;

	while(1)
	{
		if( _kbhit() )
			x = _getch();
		if (tolower(x)=='e') 
		{
			cout<<"preparing to exit"<<endl;
			s->_log.WriteServerEnd ();
			// Critical section
			s->_dataLock.RequestObj();
			
			cout<<"Bye"<<endl;
			ExitProcess(0);

			s->_dataLock.ReleaseObj();
			// End of critical section
		}
		Sleep(1000);
	}
}
/*---------------------------------------------------------------------------*/
SOCKET Server::CreateSocketAndListen() throw (ServerError)
{
	SOCKET sock; 
	// Must be done at the beginning of every WinSock program
	// used to store information about WinSock version
	try
	{ 
		initWinsockDll();
	}
	catch (ServerError e)
	{
		throw e;
	}
	
	sock= socket (AF_INET, SOCK_STREAM, 0); 

	sockaddr_in addr; // the address structure for a TCP socket
	addr.sin_family = AF_INET;      // Address family Internet
	addr.sin_port = htons (_port);   // Assign port to this socket
	addr.sin_addr.s_addr = htonl (INADDR_ANY);   // No destination
	if (bind(sock, (LPSOCKADDR)&addr, sizeof(addr)) == SOCKET_ERROR)
	{ // error
		WSACleanup ();  // unload WinSock
		throw ServerError(BIND_ERR);     // quit
	}
	if (listen(sock,/*SOMAXCONN*/10)==SOCKET_ERROR)
	{ // error!  unable to listen
		 WSACleanup ();
		 throw ServerError(LISTEN_ERR);
	}
	// listening
	return sock;
}
/*---------------------------------------------------------------------------*/
void Server::Start() throw (ServerError)
{
	
	fd_set rfds;				// For select system call
	struct timeval timeout;		// For select system call
	ifstream recovery_file_read; // Recovery file for reading
	ofstream recovery_file_write; // Rcovery file for writing
	SOCKET client;					// Accepted client
	AgentList::agent_list quited, 	// List of agents that are down
						  exist;  // List of agents currently in the system
	ThreadParameters *params = NULL; // Parameters for client handling thread

	// Read recovery file
	recovery_file_read.open(RECOVERY_FILE,ios::binary);
	read(recovery_file_read);
	recovery_file_read.close();

	// Init socket and listen
	cout<<"Starting Center Server"<<endl;
	_log.WriteServerStart ();
	try 
	{
	_MasterSock=CreateSocketAndListen();
	}
	catch (ServerError e)
	{
		throw e;
	}
  
	cout<<"Center Started and listening"<<endl;
	cout<<"Type 'e' to exit"<<endl;
	// Start exit request listening thread
	_beginthread(HandleScreenThread,NULL,(void *)this);	

	_last_switch = time(NULL);
	// Main loop
	while(true)
	{
		int ret; // Return value from select
		
		// Init select parameters
		timeout.tv_sec=SLEEP_TIME;
		timeout.tv_usec=0;
		FD_ZERO(&rfds);
		FD_SET(_MasterSock,&rfds);

		ret=select(_MasterSock+1,&rfds,NULL,NULL,&timeout);
		
		// Check for switching time
		if ((time(NULL) - _last_switch) > _switchingTime)	
		{
			// Critical section
			_dataLock.RequestObj();

			// Perform switching

			// Compute quited and down agents
			quited=_agentsList.ComputeQuited();
			// Compute existing agents
			exist =_agentsList.ComputeExist();
			_agentsList.updateList();  // Switch!
			_last_switch = time(NULL);
			// Write recovery
			recovery_file_write.open(RECOVERY_FILE,ios::binary);
			write(recovery_file_write);
			recovery_file_write.close();

			_dataLock.ReleaseObj();
			// End of critical section

			// Write to log
			_log.WriteSwitch ();
			_log.WriteExist(exist.begin(),exist.end());
			_log.WriteQuits(quited.begin(),quited.end());
		}
		
		if (ret==0) 
		// No agent is trying to connect
			continue;

		// Critical section
		_dataLock.RequestObj();


		if (_num_of_threads>=MAX_THREADS)
		// Accept only if there are less the MAX_THREADS threads
		{
				_dataLock.ReleaseObj();
				continue;
		}
		
		_num_of_threads++;

		_dataLock.ReleaseObj();	
		// End of critical section

		// Accept a new agent
		client=accept (_MasterSock, NULL, 0); 
		if (client== INVALID_SOCKET)
		{ // error accepting connection
			WSACleanup ();
			closesocket(_MasterSock);
			throw ServerError(ACCEPT_ERR);
		}
		else
		{ 
			// client connected successfully
			// start a thread that will communicate with client

			// Make parameters struct for thread
			params = NULL;
			params = new ThreadParameters;
			if (params == NULL)
				throw ServerError(MEM_ERR);
			params->_server = this;
			params->_socket = client;
			_beginthread(HandleSingleClientThread,NULL,(void *)params);
		}
	}
}
/*---------------------------------------------------------------------------*/
void Server::read (ifstream &is_recovery)
{

	char cur_dir_ini[2048]; // Current directory

	GetCurrentDirectory(2048,cur_dir_ini);
	strcat(cur_dir_ini,INI_FILE_FULL);

	// Read data from INI file
	_port=
		GetPrivateProfileInt("CENTER PARAMETERS","port",PORT,
							 cur_dir_ini);
	_params.tick_time = 
		GetPrivateProfileInt("AGENT PARAMETERS","tick time (msec)",10000,
							 cur_dir_ini);
	_params.reply_time=
		GetPrivateProfileInt("AGENT PARAMETERS","reply time (msec)",10000,
							 cur_dir_ini);

	_params.keepalive_time = 
		GetPrivateProfileInt("AGENT PARAMETERS","keepalive time (msec)",30000,
							 cur_dir_ini);
	_params.route_length = 
		GetPrivateProfileInt("AGENT PARAMETERS","route length ",100,
							 cur_dir_ini);

	// Calc switching time keepalive time * 2
	_switchingTime= _params.keepalive_time *2;
	_switchingTime/=1000; // Make it in seconds

	if (is_recovery.is_open())
	// Recovery file exits, read data!
		_agentsList.read(is_recovery);
}
/*__________________________________________________________________________*/
void Server::write (ofstream &os) const
{
	_agentsList.write(os);
}
/*---------------------------------------------------------------------------*/
void Server::initWinsockDll() throw (ServerError)
{
	WORD wVersionRequested;
	WSADATA wsaData;
	int err; 

	wVersionRequested = MAKEWORD(2, 2); 
	err = WSAStartup(wVersionRequested, &wsaData);

	if (err != 0) 
	{
		/* Tell the user that we could not find a usable WinSock DLL. */
		throw ServerError(WSA_ERR);
	} 

	/* Confirm that the WinSock DLL supports 2.2.        */
	/* Note that if the DLL supports versions greater    */
	/* than 2.2 in addition to 2.2, it will still return */
	/* 2.2 in wVersion since that is the version we      */
	/* requested.                                        */ 
	if (LOBYTE(wsaData.wVersion) != 2 ||
        HIBYTE(wsaData.wVersion) != 2 ) 
	{
		cout<<"LOBYTE = "<<LOBYTE(wsaData.wVersion)<<endl;
		cout<<"HIBYTE = "<<HIBYTE(wsaData.wVersion)<<endl;

		/* Tell the user that we could not find a usable WinSock DLL.*/
		WSACleanup();
		throw ServerError(WSA_ERR);
	}
	/* The WinSock DLL is acceptable. Proceed. */ 
}
/*---------------------------------------------------------------------------*/
void Server::HandleSingleClient(SOCKET clientSock) throw (ServerError)
{
	char buffer[MAX_REQUEST_SIZE]; // Client request
	unsigned int MAGIC; // msg MAGIC word 
	unsigned char header; // msg header
	char *buf_pos = buffer;
	unsigned int recvd=0;	// Return value from recv

	// Read request from client
	if ((recvd=recv(clientSock, buffer,MAX_REQUEST_SIZE, 0)) == -1)
	{
		throw ServerError(RECEIVE_ERR);
	}
	
	// Check if size of request is good
	if ((recvd!=MAX_REQUEST_SIZE) && (recvd!=QUIT_REQUEST_SIZE))
	{
		throw ServerError(UNRECOGNIZED_REQUEST);
	}

	// Read and check magic
	memcpy(&MAGIC,buf_pos,MAGIC_SIZE);
	buf_pos += MAGIC_SIZE;
	
	if (MAGIC!=MAGIC_HEADER)	
	{
		throw ServerError(UNKNOWN_MAGIC);
	}
	// Read header of request
	memcpy(&header,buf_pos,HEADER_SIZE);
	buf_pos += HEADER_SIZE;

	// Handle request
	switch(header) 
	{
		case GET_AGENTS_CODE: //get list request
			{
				try
				{	
					HandleGetListRequest(buf_pos,clientSock);
				}
				catch (ServerError e)
				{	
					throw e;
				}
			}
			break;
			case 0xFF: // quit request
			{
				HandleQuitRequest (buf_pos);
				
			}
			break;
			default:
			{
				throw ServerError(UNRECOGNIZED_REQUEST);
			}
			break;
		};
}
/*---------------------------------------------------------------------------*/
void Server::HandleGetListRequest (char *buf_pos, SOCKET clientSock) throw (ServerError)
{
	struct AGENT temp_client; // Current client
	struct CENTER_PARAMS temp_params; // Local copy of server params
	AgentList::agent_list list_temp; // Local copy of agents list
	AgentList::list_const_iter iter; // Iterator for agents list
	bool new_agent=false;

	// Read agent data
	memcpy(&temp_client,buf_pos,sizeof(struct AGENT));
	buf_pos += sizeof(struct AGENT);
	
	// Make a new client from agent-data
	Client client(temp_client);
	
	// Critical section
	_dataLock.RequestObj();	

	// Update the client data in the agents list
	new_agent= _agentsList.updateClient(client);
	
	// Copy parameters 
	_params.center_time = GetGlobalTime();
	_params.list_use_time = _last_switch + (_params.keepalive_time / 1000); 
	temp_params = _params;
	
	// Copy the list				
	list_temp = _agentsList.getList();

	_dataLock.ReleaseObj();			
	// End of critical section
	
	if (new_agent)
	// Write to log (register event)
		_log.WriteRegister(temp_client.address,true);				

	// Create reply
	RefCountBuffer sendBuffer(MAGIC_SIZE +
		HEADER_SIZE + 
		sizeof(struct CENTER_PARAMS) +
		sizeof(int) + // size of the list length
		sizeof(struct AGENT) *list_temp.size());

	buf_pos = sendBuffer.get_data();
		
	// Add magic
	memcpy(buf_pos, &MAGIC_HEADER, MAGIC_SIZE);
	buf_pos+=MAGIC_SIZE;
	// Add msg reply code
	memcpy(buf_pos, &GET_AGENTS_CODE, HEADER_SIZE);
	buf_pos+=HEADER_SIZE;
	// Add center's parameters
	memcpy(buf_pos, &temp_params, sizeof(struct CENTER_PARAMS));
	buf_pos+=sizeof(struct CENTER_PARAMS);

	int temp = list_temp.size(); // Number of agents in list
	// Add the number of agents
	memcpy(buf_pos, &temp,sizeof(temp));
	buf_pos+=sizeof(temp);
				
	// Prepare buffer with all agents
	for (iter = list_temp.begin(); iter != list_temp.end(); iter++)
	{
					
		// Make a new agent from list
		temp_client.address.ip = (*iter).second.getIp() ;
		temp_client.address.port = (*iter).second.getPort();
		memcpy(temp_client.pubkey,
			  (*iter).second.getPublicKey(),
			   PUBLIC_KEY_SIZE);
		
		strncpy(temp_client.nickname,
			(*iter).second.getNick().c_str(),
			NICKNAME_SIZE);
		// Add agent to list					
		memcpy(buf_pos, &temp_client, sizeof(temp_client));
		buf_pos+=sizeof(temp_client);
		
	}
		
	// Send agents list
	if ((temp=send(clientSock, sendBuffer.get_data(),
				   sendBuffer.get_size(), 0)) == -1) 
	{
		throw ServerError(SEND_ERR);
	}
	
}
/*---------------------------------------------------------------------------*/
void Server::HandleQuitRequest (char *buf_pos)
{
	struct ADDRESS adr; // Current agent's address	

	// Read agent address
	memcpy(&adr,buf_pos,sizeof(struct ADDRESS));
	buf_pos += sizeof(struct ADDRESS);
	// Write to log
	_log.WriteUnRegister(adr);
	// Create new client with agent-data
	Client client(adr);
	// Critical section
	_dataLock.RequestObj();
	// Unregister client
	_agentsList.removeClient(client);
	
	_dataLock.ReleaseObj();					
	// End of critical section
	
	// no reply needed
}
/*---------------------------------------------------------------------------*/