// server.h
// The main server class
/*---------------------------------------------------------------------------*/
#ifndef SERVER_H
#define SERVER_H
/*---------------------------------------------------------------------------*/
#pragma warning(disable : 4786) 
#include "agentlist.h"
#include "protocol.h"
#include <process.h>
#include <stdlib.h>
#include <time.h>
#include <winsock2.h>
#include <stdio.h>
#include <iostream>
#include <fstream>
#include "RefCountBuffer.h"
#include "ServerError.h"
#include "log.h"
#include "SyncObj.h"
using namespace std;
/*---------------------------------------------------------------------------*/
// CONSTS
/** Time for select in main loop to sleep */
const int SLEEP_TIME =1;
/** Maximum number of backlogged connections */ 
const int MAX_QUEUE=10; 
/** Server's recovery file */
const char RECOVERY_FILE[]=  "recovery.dat";
/** Server's ini file*/ 
const char INI_FILE[]=  "ini.txt"; 
/** Server's ini file*/
const char INI_FILE_FULL[]=  "\\ini.txt";
/** Maximum number of simultaneous threads */ 
const int  MAX_THREADS=5; 
/*---------------------------------------------------------------------------*/
/**
* Main software class - serving clients
*/
class Server
{
private:
	/** Main socket */
	SOCKET _MasterSock; 
	/** Port of center */
	int _port;			   
	/** Center parameters */
	CENTER_PARAMS _params;
	/** Period of time for internal update*/
	time_t _switchingTime; 
	/** Time of last switching	*/
	time_t _last_switch;   
	/** List of agents*/
	AgentList _agentsList; 
	/** Log object*/
	Log _log;			   
	/** Number of current running threads*/
	int _num_of_threads;   
	/** Lock for center parameters*/
	SyncObj _dataLock;	   

public:
	/** 
	  * See handleclient.h
      */
	friend void HandleSingleClientThread(LPVOID lpParameter);
	/** Listens on standard input and waits for "e" - exit request
	  * from user.Then, exits the proccess and kills all threads
	  */
	friend void HandleScreenThread(LPVOID lpParameter); 
	

	/** Server default constructor */
	Server() :_port(PORT), _num_of_threads(0) {}	
	/** Server default destructor*/	
	~Server()
	{
		// Close main socket
		closesocket(_MasterSock); 
		// Unload winsock.dll 
		WSACleanup();	
	}

	/** Starts the server */
	void Start()  throw (ServerError);
	
	/**Reads server's data from file */
	void read (ifstream &is_recovery);
	/**Writes server's data to file	*/
	void write (ofstream &os) const;
	
	/**Handle request of single client*/ 
	void HandleSingleClient(SOCKET clientSock) throw (ServerError);
	/**Handle get-list request*/
	void HandleGetListRequest (char *buf_pos, SOCKET clientSock) throw (ServerError);
	//Handle quit request*/
	void HandleQuitRequest (char *buf_pos);

private:
	/** Returns the system global-time*/
	time_t GetGlobalTime() { return time(0); }
	/**Initialize windows sockets*/
	void initWinsockDll() throw (ServerError);
	
	/** Create the server's main socket and listen*/
	SOCKET CreateSocketAndListen() throw (ServerError);
	
};
/*---------------------------------------------------------------------------*/
#endif